# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Action types for Lybic SDK."""

from typing import Union

# Common types
from .common import (
    PixelLength,
    FractionalLength,
    Length,
    ClientUserTakeoverAction,
    ScreenshotAction,
    WaitAction,
    FinishedAction,
    FailedAction,
    CommonAction,
)

# Computer use actions
from .computer import (
    MouseClickAction,
    MouseTripleClickAction,
    MouseDoubleClickAction,
    MouseMoveAction,
    MouseScrollAction,
    MouseDragAction,
    KeyboardTypeAction,
    KeyboardHotkeyAction,
    KeyDownAction,
    KeyUpAction,
    ComputerUseAction,
)

# Touch actions
from .touch import (
    TouchTapAction,
    TouchDragAction,
    TouchSwipeAction,
    TouchLongPressAction,
)

# Android actions
from .android import (
    AndroidBackAction,
    AndroidHomeAction,
)

# OS actions
from .os import (
    OsStartAppAction,
    OsStartAppByNameAction,
    OsCloseAppAction, OsListAppsAction,
)

# Mobile-specific actions remain with "mobile:" prefix for backward compatibility
MobileUseAction = Union[
    ScreenshotAction,        # generalActionScreenshotSchema
    WaitAction,              # generalActionWaitSchema
    FinishedAction,          # generalActionFinishedSchema
    FailedAction,            # generalActionFailedSchema
    ClientUserTakeoverAction, # generalActionUserTakeoverSchema

    KeyboardTypeAction, # generalActionKeyboardTypeSchema
    KeyboardHotkeyAction, # generalActionKeyboardHotkeySchema

    TouchTapAction, # mobileUseActionTapSchema
    TouchDragAction,  # mobileUseActionDragSchema
    TouchSwipeAction, # mobileUseActionSwipeSchema
    TouchLongPressAction, # mobileUseActionLongPressSchema
    AndroidBackAction, # mobileUseActionPressBackSchema
    AndroidHomeAction, # mobileUseActionPressHomeSchema

    OsStartAppAction, # mobileUseActionStartAppSchema
    OsStartAppByNameAction, # mobileUseActionStartAppByNameSchema
    OsCloseAppAction, # mobileUseActionCloseAppSchema
    OsListAppsAction, # mobileUseActionListAppsSchema
]


Action = Union[ComputerUseAction, MobileUseAction]


__all__ = [
    # Common types
    "PixelLength",
    "FractionalLength",
    "Length",
    "ClientUserTakeoverAction",
    "ScreenshotAction",
    "WaitAction",
    "FinishedAction",
    "FailedAction",
    "CommonAction",

    # Computer use actions
    "MouseClickAction",
    "MouseTripleClickAction",
    "MouseDoubleClickAction",
    "MouseMoveAction",
    "MouseScrollAction",
    "MouseDragAction",
    "KeyboardTypeAction",
    "KeyboardHotkeyAction",
    "KeyDownAction",
    "KeyUpAction",
    "ComputerUseAction",

    # Touch actions
    "TouchTapAction",
    "TouchDragAction",
    "TouchSwipeAction",
    "TouchLongPressAction",

    # Android actions
    "AndroidBackAction",
    "AndroidHomeAction",

    # OS actions
    "OsStartAppAction",
    "OsStartAppByNameAction",
    "OsCloseAppAction",
    "OsListAppsAction",

    # Union types
    "MobileUseAction",
    "Action",
]



"""Helper functions for deprecating parts of the Lybic API.

This module was adapted from matplotlibs _api/deprecation.py module:

https://github.com/matplotlib/matplotlib/blob/main/lib/matplotlib/_api/deprecation.py

.. warning::

    This module is for internal use only.  Do not use it in your own code.
    We may change the API at any time with no warning.
"""
# pylint: skip-file
import contextlib
import functools
import inspect
import warnings
from collections.abc import Generator
from typing import (
    Any,
    Callable,
    TypeVar,
    Union,
    cast,
)

from pydantic.fields import FieldInfo
from pydantic.v1.fields import FieldInfo as FieldInfoV1
from typing_extensions import ParamSpec

from lybic._api.internal import is_caller_internal


class LybicDeprecationWarning(DeprecationWarning):
    """A class for issuing deprecation warnings for Lybic users."""


class LybicPendingDeprecationWarning(PendingDeprecationWarning):
    """A class for issuing deprecation warnings for Lybic users."""


# PUBLIC API


# Last Any should be FieldInfoV1 but this leads to circular imports
T = TypeVar("T", bound=Union[type, Callable[..., Any], Any])


def _validate_deprecation_params(
    removal: str,
    alternative: str,
    alternative_import: str,
    *,
    pending: bool,
) -> None:
    """Validate the deprecation parameters."""
    if pending and removal:
        msg = "A pending deprecation cannot have a scheduled removal"
        raise ValueError(msg)
    if alternative and alternative_import:
        msg = "Cannot specify both alternative and alternative_import"
        raise ValueError(msg)

    if alternative_import and "." not in alternative_import:
        msg = (
            "alternative_import must be a fully qualified module path. Got "
            f" {alternative_import}"
        )
        raise ValueError(msg)


def deprecated(
    since: str,
    *,
    message: str = "",
    name: str = "",
    alternative: str = "",
    alternative_import: str = "",
    pending: bool = False,
    obj_type: str = "",
    addendum: str = "",
    removal: str = "",
    package: str = "",
) -> Callable[[T], T]:
    """Decorator to mark a function, a class, or a property as deprecated.

    When deprecating a classmethod, a staticmethod, or a property, the
    ``@deprecated`` decorator should go *under* ``@classmethod`` and
    ``@staticmethod`` (i.e., `deprecated` should directly decorate the
    underlying callable), but *over* ``@property``.

    When deprecating a class ``C`` intended to be used as a base class in a
    multiple inheritance hierarchy, ``C`` *must* define an ``__init__`` method
    (if ``C`` instead inherited its ``__init__`` from its own base class, then
    ``@deprecated`` would mess up ``__init__`` inheritance when installing its
    own (deprecation-emitting) ``C.__init__``).

    Parameters are the same as for `warn_deprecated`, except that *obj_type*
    defaults to 'class' if decorating a class, 'attribute' if decorating a
    property, and 'function' otherwise.

    Args:
        since : str
            The release at which this API became deprecated.
        message : str, optional
            Override the default deprecation message. The %(since)s,
            %(name)s, %(alternative)s, %(obj_type)s, %(addendum)s,
            and %(removal)s format specifiers will be replaced by the
            values of the respective arguments passed to this function.
        name : str, optional
            The name of the deprecated object.
        alternative : str, optional
            An alternative API that the user may use in place of the
            deprecated API. The deprecation warning will tell the user
            about this alternative if provided.
        alternative_import: str, optional
            An alternative import that the user may use instead.
        pending : bool, optional
            If True, uses a PendingDeprecationWarning instead of a
            DeprecationWarning. Cannot be used together with removal.
        obj_type : str, optional
            The object type being deprecated.
        addendum : str, optional
            Additional text appended directly to the final message.
        removal : str, optional
            The expected removal version. With the default (an empty
            string), a removal version is automatically computed from
            since. Set to other Falsy values to not schedule a removal
            date. Cannot be used together with pending.
        package: str, optional
            The package of the deprecated object.

    Examples:

        .. code-block:: python

            @deprecated('1.4.0')
            def the_function_to_deprecate():
                pass

    """
    _validate_deprecation_params(
        removal, alternative, alternative_import, pending=pending
    )

    def deprecate(
        obj: T,
        *,
        _obj_type: str = obj_type,
        _name: str = name,
        _message: str = message,
        _alternative: str = alternative,
        _alternative_import: str = alternative_import,
        _pending: bool = pending,
        _addendum: str = addendum,
        _package: str = package,
    ) -> T:
        """Implementation of the decorator returned by `deprecated`."""

        def emit_warning() -> None:
            """Emit the warning."""
            warn_deprecated(
                since,
                message=_message,
                name=_name,
                alternative=_alternative,
                alternative_import=_alternative_import,
                pending=_pending,
                obj_type=_obj_type,
                addendum=_addendum,
                removal=removal,
                package=_package,
            )

        warned = False

        def warning_emitting_wrapper(*args: Any, **kwargs: Any) -> Any:
            """Wrapper for the original wrapped callable that emits a warning.

            Args:
                *args: The positional arguments to the function.
                **kwargs: The keyword arguments to the function.

            Returns:
                The return value of the function being wrapped.
            """
            nonlocal warned
            if not warned and not is_caller_internal():
                warned = True
                emit_warning()
            return wrapped(*args, **kwargs)

        async def awarning_emitting_wrapper(*args: Any, **kwargs: Any) -> Any:
            """Same as warning_emitting_wrapper, but for async functions."""
            nonlocal warned
            if not warned and not is_caller_internal():
                warned = True
                emit_warning()
            return await wrapped(*args, **kwargs)

        _package = _package or obj.__module__.split(".")[0].replace("_", "-")

        if isinstance(obj, type):
            if not _obj_type:
                _obj_type = "class"
            wrapped = obj.__init__  # type: ignore[misc]
            _name = _name or obj.__qualname__
            old_doc = obj.__doc__

            def finalize(wrapper: Callable[..., Any], new_doc: str) -> T:  # noqa: ARG001
                """Finalize the deprecation of a class."""
                # Can't set new_doc on some extension objects.
                with contextlib.suppress(AttributeError):
                    obj.__doc__ = new_doc

                def warn_if_direct_instance(
                    self: Any, *args: Any, **kwargs: Any
                ) -> Any:
                    """Warn that the class is in beta."""
                    nonlocal warned
                    if not warned and type(self) is obj and not is_caller_internal():
                        warned = True
                        emit_warning()
                    return wrapped(self, *args, **kwargs)

                obj.__init__ = functools.wraps(obj.__init__)(  # type: ignore[misc]
                    warn_if_direct_instance
                )
                return cast("T", obj)

        elif isinstance(obj, FieldInfoV1):
            wrapped = None
            if not _obj_type:
                _obj_type = "attribute"
            if not _name:
                msg = f"Field {obj} must have a name to be deprecated."
                raise ValueError(msg)
            old_doc = obj.description

            def finalize(wrapper: Callable[..., Any], new_doc: str) -> T:  # noqa: ARG001
                kwargs = {
                    "default": obj.default,
                    "default_factory": obj.default_factory,
                    "alias": obj.alias,
                    "title": obj.title,
                    "const": obj.const,
                    "gt": obj.gt,
                    "ge": obj.ge,
                    "lt": obj.lt,
                    "le": obj.le,
                    "multiple_of": obj.multiple_of,
                    "min_items": obj.min_items,
                    "max_items": obj.max_items,
                    "min_length": obj.min_length,
                    "max_length": obj.max_length,
                    "regex": obj.regex,
                }
                kwargs.update(getattr(obj, "extra", {}))
                kwargs["description"] = new_doc
                return cast(
                    "T",
                    FieldInfoV1(**kwargs),
                )

        elif isinstance(obj, FieldInfo):
            wrapped = None
            if not _obj_type:
                _obj_type = "attribute"
            if not _name:
                msg = f"Field {obj} must have a name to be deprecated."
                raise ValueError(msg)
            old_doc = obj.description

            def finalize(wrapper: Callable[..., Any], new_doc: str) -> T:  # noqa: ARG001
                kwargs = {
                    attr: getattr(obj, attr)
                    for attr in getattr(obj, "_attributes_set", set())
                    if hasattr(obj, attr)
                }
                kwargs["description"] = new_doc
                return cast(
                    "T",
                    FieldInfo(**kwargs),
                )

        elif isinstance(obj, property):
            if not _obj_type:
                _obj_type = "attribute"
            wrapped = None
            _name = _name or cast("Union[type, Callable]", obj.fget).__qualname__
            old_doc = obj.__doc__

            class _DeprecatedProperty(property):
                """A deprecated property."""

                def __init__(
                    self,
                    fget: Union[Callable[[Any], Any], None] = None,
                    fset: Union[Callable[[Any, Any], None], None] = None,
                    fdel: Union[Callable[[Any], None], None] = None,
                    doc: Union[str, None] = None,
                ) -> None:
                    super().__init__(fget, fset, fdel, doc)
                    self.__orig_fget = fget
                    self.__orig_fset = fset
                    self.__orig_fdel = fdel

                def __get__(
                    self, instance: Any, owner: Union[type, None] = None
                ) -> Any:
                    if instance is not None or owner is not None:
                        emit_warning()
                    if self.fget is None:
                        return None
                    return self.fget(instance)

                def __set__(self, instance: Any, value: Any) -> None:
                    if instance is not None:
                        emit_warning()
                    if self.fset is not None:
                        self.fset(instance, value)

                def __delete__(self, instance: Any) -> None:
                    if instance is not None:
                        emit_warning()
                    if self.fdel is not None:
                        self.fdel(instance)

                def __set_name__(self, owner: Union[type, None], set_name: str) -> None:
                    nonlocal _name
                    if _name == "<lambda>":
                        _name = set_name

            def finalize(wrapper: Callable[..., Any], new_doc: str) -> T:  # noqa: ARG001
                """Finalize the property."""
                return cast(
                    "T",
                    _DeprecatedProperty(
                        fget=obj.fget, fset=obj.fset, fdel=obj.fdel, doc=new_doc
                    ),
                )

        else:
            _name = _name or cast("Union[type, Callable]", obj).__qualname__
            if not _obj_type:
                # edge case: when a function is within another function
                # within a test, this will call it a "method" not a "function"
                _obj_type = "function" if "." not in _name else "method"
            wrapped = obj
            old_doc = wrapped.__doc__

            def finalize(wrapper: Callable[..., Any], new_doc: str) -> T:
                """Wrap the wrapped function using the wrapper and update the docstring.

                Args:
                    wrapper: The wrapper function.
                    new_doc: The new docstring.

                Returns:
                    The wrapped function.
                """
                wrapper = functools.wraps(wrapped)(wrapper)
                wrapper.__doc__ = new_doc
                return cast("T", wrapper)

        old_doc = inspect.cleandoc(old_doc or "").strip("\n")

        # old_doc can be None
        if not old_doc:
            old_doc = ""

        # Modify the docstring to include a deprecation notice.
        if (
            _alternative
            and _alternative.split(".")[-1].lower() == _alternative.split(".")[-1]
        ):
            _alternative = f":meth:`~{_alternative}`"
        elif _alternative:
            _alternative = f":class:`~{_alternative}`"

        if (
            _alternative_import
            and _alternative_import.split(".")[-1].lower()
            == _alternative_import.split(".")[-1]
        ):
            _alternative_import = f":meth:`~{_alternative_import}`"
        elif _alternative_import:
            _alternative_import = f":class:`~{_alternative_import}`"

        components = [
            _message,
            f"Use {_alternative} instead." if _alternative else "",
            f"Use ``{_alternative_import}`` instead." if _alternative_import else "",
            _addendum,
        ]
        details = " ".join([component.strip() for component in components if component])
        package = _package or (
            _name.split(".")[0].replace("_", "-") if "." in _name else None
        )
        if removal:
            if removal.startswith("1.") and package and package.startswith("lybic"):
                removal_str = f"It will not be removed until {package}=={removal}."
            else:
                removal_str = f"It will be removed in {package}=={removal}."
        else:
            removal_str = ""
        new_doc = f"""\
.. deprecated:: {since} {details} {removal_str}

{old_doc}\
"""

        if inspect.iscoroutinefunction(obj):
            return finalize(awarning_emitting_wrapper, new_doc)
        return finalize(warning_emitting_wrapper, new_doc)

    return deprecate


@contextlib.contextmanager
def suppress_lybic_deprecation_warning() -> Generator[None, None, None]:
    """Context manager to suppress LybicDeprecationWarning."""
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", LybicDeprecationWarning)
        warnings.simplefilter("ignore", LybicPendingDeprecationWarning)
        yield


def warn_deprecated(
    since: str,
    *,
    message: str = "",
    name: str = "",
    alternative: str = "",
    alternative_import: str = "",
    pending: bool = False,
    obj_type: str = "",
    addendum: str = "",
    removal: str = "",
    package: str = "",
) -> None:
    """Display a standardized deprecation.

    Arguments:
        since : str
            The release at which this API became deprecated.
        message : str, optional
            Override the default deprecation message. The %(since)s,
            %(name)s, %(alternative)s, %(obj_type)s, %(addendum)s,
            and %(removal)s format specifiers will be replaced by the
            values of the respective arguments passed to this function.
        name : str, optional
            The name of the deprecated object.
        alternative : str, optional
            An alternative API that the user may use in place of the
            deprecated API. The deprecation warning will tell the user
            about this alternative if provided.
        alternative_import: str, optional
            An alternative import that the user may use instead.
        pending : bool, optional
            If True, uses a PendingDeprecationWarning instead of a
            DeprecationWarning. Cannot be used together with removal.
        obj_type : str, optional
            The object type being deprecated.
        addendum : str, optional
            Additional text appended directly to the final message.
        removal : str, optional
            The expected removal version. With the default (an empty
            string), a removal version is automatically computed from
            since. Set to other Falsy values to not schedule a removal
            date. Cannot be used together with pending.
        package: str, optional
            The package of the deprecated object.
    """
    if not pending:
        if not removal:
            removal = f"in {removal}" if removal else "within ?? minor releases"
            msg = (
                f"Need to determine which default deprecation schedule to use. "
                f"{removal}"
            )
            raise ValueError(msg)
        removal = f"in {removal}"

    if not message:
        message = ""
        package_ = (
            package or name.split(".")[0].replace("_", "-")
            if "." in name
            else "Lybic"
        )

        if obj_type:
            message += f"The {obj_type} `{name}`"
        else:
            message += f"`{name}`"

        if pending:
            message += " will be deprecated in a future version"
        else:
            message += f" was deprecated in {package_} {since}"

            if removal:
                message += f" and will be removed {removal}"

        if alternative_import:
            alt_package = alternative_import.split(".")[0].replace("_", "-")
            if alt_package == package_:
                message += f". Use {alternative_import} instead."
            else:
                alt_module, alt_name = alternative_import.rsplit(".", 1)
                message += (
                    f". An updated version of the {obj_type} exists in the "
                    f"{alt_package} package and should be used instead. To use it run "
                    f"`pip install -U {alt_package}` and import as "
                    f"`from {alt_module} import {alt_name}`."
                )
        elif alternative:
            message += f". Use {alternative} instead."

        if addendum:
            message += f" {addendum}"

    warning_cls = (
        LybicPendingDeprecationWarning if pending else LybicDeprecationWarning
    )
    warning = warning_cls(message)
    warnings.warn(warning, category=LybicDeprecationWarning, stacklevel=4)


def surface_lybic_deprecation_warnings() -> None:
    """Unmute Lybic deprecation warnings."""
    warnings.filterwarnings(
        "default",
        category=LybicPendingDeprecationWarning,
    )

    warnings.filterwarnings(
        "default",
        category=LybicDeprecationWarning,
    )


_P = ParamSpec("_P")
_R = TypeVar("_R")


def rename_parameter(
    *,
    since: str,
    removal: str,
    old: str,
    new: str,
) -> Callable[[Callable[_P, _R]], Callable[_P, _R]]:
    """Decorator indicating that parameter *old* of *func* is renamed to *new*.

    The actual implementation of *func* should use *new*, not *old*.  If *old*
    is passed to *func*, a DeprecationWarning is emitted, and its value is
    used, even if *new* is also passed by keyword.

    Example:

        .. code-block:: python

            @_api.rename_parameter("3.1", "bad_name", "good_name")
            def func(good_name): ...

    """

    def decorator(f: Callable[_P, _R]) -> Callable[_P, _R]:
        @functools.wraps(f)
        def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> _R:
            if new in kwargs and old in kwargs:
                msg = f"{f.__name__}() got multiple values for argument {new!r}"
                raise TypeError(msg)
            if old in kwargs:
                warn_deprecated(
                    since,
                    removal=removal,
                    message=f"The parameter `{old}` of `{f.__name__}` was "
                    f"deprecated in {since} and will be removed "
                    f"in {removal} Use `{new}` instead.",
                )
                kwargs[new] = kwargs.pop(old)
            return f(*args, **kwargs)

        return wrapper

    return decorator



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""project.py provides the Project manager ability to use"""
from typing import overload, TYPE_CHECKING

from lybic import dto

if TYPE_CHECKING:
    from lybic.lybic import LybicClient

class Project:
    """
    Projects class are used to organize Projects.
    """
    def __init__(self, client: "LybicClient"):
        self.client = client

    async def list(self) -> dto.ListProjectsResponseDto:
        """
        List all projects in the organization.
        """
        self.client.logger.debug("Listing projects request")
        response = await self.client.request("GET", f"/api/orgs/{self.client.org_id}/projects")
        self.client.logger.debug("Listing projects response: %s", response.text)
        return dto.ListProjectsResponseDto.model_validate_json(response.text)

    @overload
    async def create(self, data: dto.CreateProjectDto) -> dto.SingleProjectResponseDto: ...

    @overload
    async def create(self, **kwargs) -> dto.SingleProjectResponseDto: ...

    async def create(self, *args, **kwargs) -> dto.SingleProjectResponseDto:
        """
        Create a new project.
        """
        if args and isinstance(args[0], dto.CreateProjectDto):
            data = args[0]
        else:
            data = dto.CreateProjectDto(**kwargs)
        self.client.logger.debug("Creating project request with data: %s", data)
        response = await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/projects", json=data.model_dump(exclude_none=True))
        self.client.logger.debug("Create project response: %s", response.text)
        return dto.SingleProjectResponseDto.model_validate_json(response.text)

    async def delete(self, project_id: str) -> None:
        """
        Delete a project.
        """
        self.client.logger.debug("Deleting project request with project_id: %s", project_id)
        await self.client.request("DELETE", f"/api/orgs/{self.client.org_id}/projects/{project_id}")



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""Agentic Lybic Restful API models"""
from typing import Optional
from enum import Enum, unique

from pydantic import BaseModel

@unique
class SandboxOS(str, Enum):
    """Sandbox os type"""
    WINDOWS = "WINDOWS"
    LINUX = "LINUX"
    ANDROID = "ANDROID"

@unique
class InstanceMode(str, Enum):
    """Agentic Lybic task instance executing mode"""
    NORMAL = "NORMAL"
    FAST = "FAST"

@unique
class TaskStatus(str, Enum):
    """Agentic Lybic task status"""
    PENDING = "PENDING"
    RUNNING = "RUNNING"
    SUCCESS = "SUCCESS"
    FAILURE = "FAILURE"
    NOT_FOUND = "NOT_FOUND"

class AgentInfo(BaseModel):
    """Agentic Lybic agent info"""
    version: str
    maxConcurrentTasks: Optional[int] = None
    logLevel: Optional[str] = None
    domain: Optional[str] = None
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"
        exclude_none = True

class Sandbox(BaseModel):
    """sandbox info for task executing platform"""
    id: str
    name: str
    description: str
    shapeName: str
    hardwareAcceleratedEncoding: bool
    os: SandboxOS
    virtualization: str
    architecture: str
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"
        exclude_none = True

class Authorization(BaseModel):
    """Lybic API authorization info"""
    apiEndpoint: str
    orgID: Optional[str] = None
    apiKey: Optional[str] = None
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"
        exclude_none = True

class LLMConfig(BaseModel):
    """LLM model config"""
    modelName: str
    provider: str
    apiEndpoint: str
    apiKey: Optional[str] = None
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"
        exclude_none = True

class StageModelConfig(BaseModel):
    """Stage model config"""
    webSearchEngine: Optional[str] = None
    contextFusionModel: Optional[LLMConfig] = None
    subtaskPlannerModel: Optional[LLMConfig] = None
    trajReflectorModel: Optional[LLMConfig] = None
    memoryRetrievalModel: Optional[LLMConfig] = None
    groundingModel: Optional[LLMConfig] = None
    taskEvaluatorModel: Optional[LLMConfig] = None
    actionGeneratorModel: Optional[LLMConfig] = None
    actionGeneratorWithTakeoverModel: Optional[LLMConfig] = None
    fastActionGeneratorModel: Optional[LLMConfig] = None
    fastActionGeneratorWithTakeoverModel: Optional[LLMConfig] = None
    dagTranslatorModel: Optional[LLMConfig] = None
    embeddingModel: Optional[LLMConfig] = None
    queryFormulatorModel: Optional[LLMConfig] = None
    narrativeSummarizationModel: Optional[LLMConfig] = None
    textSpanModel: Optional[LLMConfig] = None
    episodeSummarizationModel: Optional[LLMConfig] = None
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"
        exclude_none = True

class CommonConfig(BaseModel):
    """Agentic Lybic common config"""
    id: str
    backend: str
    mode: Optional[InstanceMode] = None
    steps: Optional[int] = None
    taskTimeout: Optional[int] = None
    authorizationInfo: Optional[Authorization] = None
    stageModelConfig: Optional[StageModelConfig] = None
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"
        exclude_none = True

class RunAgentInstructionRequest(BaseModel):
    """Agentic Lybic run agent instruction request"""
    instruction: str
    sandbox: Optional[Sandbox] = None
    runningConfig: Optional[CommonConfig] = None
    class Config:
        """
        Configuration for Pydantic model.
        """
        exclude_none = True

class RunAgentInstructionAsyncResponse(BaseModel):
    """Agentic Lybic run agent instruction async response"""
    taskId: str
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"

class QueryTaskStatusResponse(BaseModel):
    """Agentic Lybic query task status response"""
    taskId: str
    status: TaskStatus
    message: Optional[str] = None
    result: Optional[str] = None
    sandbox: Optional[Sandbox] = None
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"
        exclude_none = True

class TaskStream(BaseModel):
    """Agentic Lybic task stream"""
    taskId: str
    stage: str
    message: str
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"

class SetCommonConfigResponse(BaseModel):
    """Agentic Lybic set common config response"""
    success: bool
    id: str
    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""mcp.py: MCP client for lybic MCP(Model Context Protocol) and Restful Interface API."""
import asyncio
from typing import overload, TYPE_CHECKING

import httpx

from lybic import dto
from lybic._api import deprecated

if TYPE_CHECKING:
    from lybic.lybic import LybicClient

# pylint: disable=invalid-name
try:
    from mcp import ClientSession
    from mcp.client.streamable_http import streamablehttp_client
    from mcp.types import CallToolResult
    MCP_INSTALLED = True
except ImportError:
    MCP_INSTALLED = False
    ClientSession = object
    streamablehttp_client = object
    CallToolResult = object

class Mcp:
    """Mcp is an async client for lybic MCP(Model Context Protocol) and Restful Interface API."""
    def __init__(self, client: "LybicClient"):
        """
        Init MCP client with lybic client

        :param client: LybicClient
        """
        self.client = client

    async def list(self) -> dto.ListMcpServerResponse:
        """
        List all MCP servers in the organization

        :return:
        """
        self.client.logger.debug("List MCP servers request")
        response = await self.client.request(
            "GET",
            f"/api/orgs/{self.client.org_id}/mcp-servers")
        self.client.logger.debug(f"List MCP servers response: {response.text}")
        return dto.ListMcpServerResponse.model_validate_json(response.text)

    @overload
    async def create(self, data: dto.CreateMcpServerDto) -> dto.McpServerResponseDto: ...

    @overload
    async def create(self, **kwargs) -> dto.McpServerResponseDto: ...

    async def create(self, *args, **kwargs) -> dto.McpServerResponseDto:
        """
        Create a mcp server

        :param data:
        :return:
        """
        if args and isinstance(args[0], dto.CreateMcpServerDto):
            data = args[0]
        elif "data" in kwargs and isinstance(kwargs["data"], dto.CreateMcpServerDto):
            data = kwargs["data"]
        else:
            data = dto.CreateMcpServerDto(**kwargs)
        self.client.logger.debug(f"Create MCP server request: {data.model_dump_json(exclude_none=True)}")
        response = await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/mcp-servers",
            json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Create MCP server response: {response.text}")
        return dto.McpServerResponseDto.model_validate_json(response.text)

    async def get_default(self) -> dto.McpServerResponseDto:
        """
        Get default mcp server

        :return:
        """
        self.client.logger.debug("Get default MCP server request")
        response = await self.client.request(
            "GET",
            f"/api/orgs/{self.client.org_id}/mcp-servers/default")
        self.client.logger.debug(f"Get default MCP server response: {response.text}")
        return dto.McpServerResponseDto.model_validate_json(response.text)

    async def delete(self, mcp_server_id: str) -> None:
        """
        Delete a mcp server

        :param mcp_server_id:
        :return:
        """
        self.client.logger.debug(f"Delete MCP server request: {mcp_server_id}")
        await self.client.request("DELETE", f"/api/orgs/{self.client.org_id}/mcp-servers/{mcp_server_id}")

    async def set_sandbox(self, mcp_server_id: str, sandbox_id: str) -> None:
        """
        Set MCP server to a specific sandbox

        :param mcp_server_id: The ID of the MCP server
        :param sandbox_id: The ID of the sandbox to connect the MCP server to
        :return: None
        """
        data = dto.SetMcpServerToSandboxResponseDto(sandboxId=sandbox_id)
        self.client.logger.debug(f"Set MCP server to sandbox request: {data.model_dump_json(exclude_none=True)}")
        await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/mcp-servers/{mcp_server_id}/sandbox",
            json=data.model_dump(exclude_none=True))

    async def call_tool_async(self,
                              mcp_server_id: str,
                              tool_name: str = "computer-use",
                              tool_args: dict = None) -> CallToolResult:
        """
        Call a tool on mcp server

        :param mcp_server_id:
        :param tool_name:
        :param tool_args:
        :return:
        """
        if not MCP_INSTALLED:
            raise ImportError("mcp is not installed. Please install it with `pip install 'lybic[mcp]'`")
        self.client.logger.debug(f"Call tool request: {tool_name} with arguments: {tool_args}")

        last_exception = None
        for attempt in range(self.client.max_retries + 1):
            try:
                async with streamablehttp_client(self.client.make_mcp_endpoint(mcp_server_id),
                                                 headers=self.client.headers,
                                                 timeout=self.client.timeout
                ) as (
                        read_stream,
                        write_stream,
                        _,
                ):
                    async with ClientSession(read_stream, write_stream) as session:
                        await session.initialize()
                        result = await session.call_tool(tool_name, tool_args)
                        self.client.logger.debug(f"Call tool response: {result.model_dump_json()}")
                        return result

            except (httpx.RequestError, httpx.HTTPStatusError) as e:
                last_exception = e
                if attempt < self.client.max_retries:
                    self.client.logger.debug(f"Call tool failed (attempt {attempt + 1}/{self.client.max_retries + 1}): {str(e)}")
                    await asyncio.sleep(2 ** attempt)
                else:
                    self.client.logger.error(f"Call tool failed after {self.client.max_retries + 1} attempts")

        raise RuntimeError(f"Failed to call tool: {last_exception}") from last_exception

# pylint: disable=useless-parent-delegation

@deprecated(
    since="0.8.0",
    removal="1.0.0",
    message="renamed: Use Mcp instead"
)
class MCP(Mcp):
    """
    MCP client for lybic MCP(Model Context Protocol) and Restful Interface API.

    According to the python naming convention, please use Class Mcp
    """
    def __init__(self, client: "LybicClient"):
        """
        Initialize MCP client

        :param client: LybicClient
        """
        super().__init__(client)



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""lybic.py is the main entry point for Lybic API."""
import asyncio
import json
from typing import Optional
import httpx

from .mcp import Mcp
from .stats import Stats
from .project import Project
from .sandbox import Sandbox
from .authentication import LybicAuth
from .base import _LybicBaseClient, _sentinel
from .exceptions import LybicAPIError, LybicInternalError
from .tools import Tools


class LybicClient(_LybicBaseClient):
    """LybicAsyncClient is a client for all Lybic API."""

    def __init__(self,
                 auth: Optional[LybicAuth] = None,
                 org_id: str = _sentinel,
                 api_key: str = _sentinel,
                 endpoint: str = _sentinel,
                 timeout: int = 10,
                 extra_headers: dict = _sentinel,
                 max_retries: int = 3,
                 ):
        """
        Init lybic client with org_id, api_key and endpoint

        :param auth:
        :param org_id:
        :param api_key:
        :param endpoint:
        :param max_retries: maximum number of retries for failed requests
        """
        super().__init__(
            auth=auth, org_id=org_id, api_key=api_key, endpoint=endpoint,
            timeout=timeout, extra_headers=extra_headers, max_retries=max_retries
        )

        self.client: httpx.AsyncClient | None = None
        self._in_context = False

        self.sandbox = Sandbox(self)
        self.project = Project(self)
        self.mcp = Mcp(self)
        self.stats = Stats(self)
        self.tools = Tools(self)

    def _ensure_client_is_open(self):
        if self.client is None:
            self.client = httpx.AsyncClient(headers=self.headers, timeout=self.timeout)
        elif self.client.is_closed:
            raise RuntimeError("The client has been closed and cannot be reused. Please create a new client instance.")

    async def __aenter__(self):
        if self._in_context:
            raise RuntimeError("Cannot re-enter context.")
        if self.client and not self.client.is_closed:
            raise RuntimeError("Cannot enter context with an already-active client.")

        self._in_context = True
        self._ensure_client_is_open()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self._in_context = False
        await self.close()

    async def close(self):
        """Close the underlying httpx.AsyncClient."""
        if self._in_context:
            return
        if self.client:
            await self.client.aclose()

    async def request(self, method: str, path: str, **kwargs) -> httpx.Response:
        """
        Make a request to Lybic Restful API

        :param method:
        :param path:
        :param kwargs:
        :return:
        :raises LybicAPIError: When API returns structured error response
        :raises LybicInternalError: When 5xx error occurs from reverse proxy
        :raises httpx.RequestError: When network-level error occurs
        """
        self._ensure_client_is_open()

        url = f"{self.endpoint}{path}"
        headers = self.headers.copy()
        if method.upper() != "POST":
            headers.pop("Content-Type", None)

        last_exception = None
        for attempt in range(self.max_retries + 1):
            try:
                response = await self.client.request(method, url, headers=headers, **kwargs)
                response.raise_for_status()
                return response
            except httpx.HTTPStatusError as e:
                # Check if this is the last attempt
                if attempt < self.max_retries:
                    self.logger.debug(f"Request failed (attempt {attempt + 1}/{self.max_retries + 1}): {e}")
                    last_exception = e
                    await asyncio.sleep(2 ** attempt)
                    continue

                # Last attempt, convert to custom exception
                self.logger.error("Request failed after %d attempts", self.max_retries + 1)

                # Check if it's a 5xx error from reverse proxy
                if e.response.status_code >= 500:
                    # Try to parse JSON response
                    try:
                        error_data = e.response.json()
                        if isinstance(error_data, dict) and "message" in error_data:
                            # Structured API error response
                            raise LybicAPIError(
                                message=error_data.get("message", "Unknown error"),
                                code=error_data.get("code"),
                                status_code=e.response.status_code,
                            ) from e
                    except (json.JSONDecodeError, ValueError):
                        # Not a JSON response, likely HTML from reverse proxy
                        pass

                    # If we got here, it's a reverse proxy error
                    raise LybicInternalError(status_code=e.response.status_code) from e

                # For 4xx errors, try to parse structured error response
                try:
                    error_data = e.response.json()
                    if isinstance(error_data, dict) and "message" in error_data:
                        raise LybicAPIError(
                            message=error_data.get("message", "Unknown error"),
                            code=error_data.get("code"),
                            status_code=e.response.status_code,
                        ) from e
                except (json.JSONDecodeError, ValueError):
                    # Not a JSON response, re-raise original exception
                    pass

                # If we couldn't parse it as a structured error, re-raise original
                raise
            except httpx.RequestError as e:
                last_exception = e
                if attempt < self.max_retries:
                    self.logger.debug(f"Request failed (attempt {attempt + 1}/{self.max_retries + 1}): {e}")
                else:
                    self.logger.error("Request failed after %d attempts", self.max_retries + 1)
                await asyncio.sleep(2 ** attempt)

        raise last_exception



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""Agentic Lybic Restful API client"""
import asyncio
import logging

import httpx

from lybic import LybicAuth
from lybic.gui_agents.models import (
    AgentInfo,
    CommonConfig,
    SetCommonConfigResponse,
    LLMConfig,
    RunAgentInstructionRequest,
    TaskStream,
    RunAgentInstructionAsyncResponse,
    QueryTaskStatusResponse
)


class Client:
    """Agentic Lybic Restful API client"""
    def __init__(self, auth: LybicAuth,timeout: int = 10,max_retries: int = 3):
        """Agentic Lybic Restful API client"""
        self.auth = auth
        self.timeout = timeout
        self.max_retries = max_retries
        self._httpclient: httpx.AsyncClient | None = None
        self._in_context = False
        self.logger = logging.getLogger(__name__)

    def _ensure_client_is_open(self):
        if self._httpclient is None:
            self._httpclient = httpx.AsyncClient(headers=self.auth.headers, timeout=self.timeout)
        elif self._httpclient.is_closed:
            raise RuntimeError("The client has been closed and cannot be reused. Please create a new client instance.")

    async def __aenter__(self):
        if self._in_context:
            raise RuntimeError("Cannot re-enter context.")
        if self._httpclient and not self._httpclient.is_closed:
            raise RuntimeError("Cannot enter context with an already-active client.")

        self._in_context = True
        self._ensure_client_is_open()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self._in_context = False
        await self.close()

    async def close(self):
        """Close the underlying httpx.AsyncClient."""
        if self._in_context:
            return
        if self._httpclient:
            await self._httpclient.aclose()

    async def _get(self, path: str) -> httpx.Response:
        """
        Make a request to Lybic Restful API

        :param path: API endpoint
        :return: httpx.Response object
        """
        self._ensure_client_is_open()

        url = f"{self.auth.agent_service_endpoint}{path}"
        headers = self.auth.headers.copy()
        headers.pop("Content-Type", None)
        last_exception = None
        for attempt in range(self.max_retries + 1):
            try:
                response = await self._httpclient.get(url, headers=headers)
                response.raise_for_status()
                return response
            except (httpx.RequestError, httpx.HTTPStatusError) as e:
                last_exception = e
                if attempt < self.max_retries:
                    self.logger.debug(f"Request failed (attempt {attempt + 1}/{self.max_retries + 1}): {e}")
                else:
                    self.logger.error("Request failed after %d attempts", self.max_retries + 1)
                await asyncio.sleep(2 ** attempt)

        raise last_exception

    async def _post(self, path: str, data: dict) -> httpx.Response:
        """
        Make a request to Lybic Restful API

        :param path: API endpoint
        :param data: request data
        :return: httpx.Response object
        """
        self._ensure_client_is_open()

        url = f"{self.auth.agent_service_endpoint}{path}"
        last_exception = None
        for attempt in range(self.max_retries + 1):
            try:
                response = await self._httpclient.post(url, headers=self.auth.headers,json= data)
                response.raise_for_status()
                return response
            except (httpx.RequestError, httpx.HTTPStatusError) as e:
                last_exception = e
                if attempt < self.max_retries:
                    self.logger.debug(f"Request failed (attempt {attempt + 1}/{self.max_retries + 1}): {e}")
                else:
                    self.logger.error("Request failed after %d attempts", self.max_retries + 1)
                await asyncio.sleep(2 ** attempt)

        raise last_exception

    async def _stream(self, path: str, data: dict | None = None):
        """
        Make a streaming request to the Lybic Restful API (SSE).

        :param path: API endpoint.
        :param data: Optional request data. If provided, a POST request is made. Otherwise, a GET request is made.
        """
        self._ensure_client_is_open()

        url = f"{self.auth.agent_service_endpoint}{path}"
        headers = self.auth.headers.copy()

        method = "POST" if data else "GET"
        request_kwargs = {"headers": headers}
        if data:
            request_kwargs["json"] = data
        else:
            # No content-type for GET requests with no body
            headers.pop("Content-Type", None)

        async with self._httpclient.stream(method, url, **request_kwargs) as response:
            response.raise_for_status()
            async for line in response.aiter_lines():
                if line.strip() and line.startswith("data:"):
                    yield line[len("data:"):].strip()

    async def get_agent_info(self) -> AgentInfo:
        """
        Get agent info
        :return:
        """
        response = await self._get("/api/agent/info")
        return AgentInfo.model_validate_json(response.text)

    async def get_global_common_config(self) -> CommonConfig:
        """
        Get global common config
        :return:
        """
        response = await self._get("/api/agent/config/global")
        return CommonConfig.model_validate_json(response.text)

    async def set_global_common_config(self, config: CommonConfig) -> SetCommonConfigResponse:
        """
        Set global common config
        :param config:
        :return:
        """
        response = await self._post("/api/agent/config/global", data=config.model_dump())
        return SetCommonConfigResponse.model_validate_json(response.text)

    async def get_common_config(self, config_id: str) -> CommonConfig:
        """
        Get common config by id
        :param config_id:
        :return:
        """
        response = await self._get(f"/api/agent/config/{config_id}")
        return CommonConfig.model_validate_json(response.text)

    async def set_global_common_llm_config(self, config: LLMConfig) -> LLMConfig:
        """
        Set global common llm config
        :param config:
        :return:
        """
        response = await self._post("/api/agent/config/global/llm", data=config.model_dump())
        return LLMConfig.model_validate_json(response.text)

    async def get_global_grounding_llm_config(self) -> LLMConfig:
        """
        Get global grounding llm config
        :return:
        """
        response = await self._get("/api/agent/config/global/grounding-llm")
        return LLMConfig.model_validate_json(response.text)

    async def set_global_grounding_llm_config(self, config: LLMConfig) -> LLMConfig:
        """
        Set global grounding llm config
        :param config:
        :return:
        """
        response = await self._post("/api/agent/config/global/grounding-llm", data=config.model_dump())
        return LLMConfig.model_validate_json(response.text)

    async def set_global_embedding_llm_config(self, config: LLMConfig) -> LLMConfig:
        """
        Set global embedding llm config
        :param config:
        :return:
        """
        response = await self._post("/api/agent/config/global/embedding-llm", data=config.model_dump())
        return LLMConfig.model_validate_json(response.text)

    async def run_agent_instruction(self, request: RunAgentInstructionRequest):
        """
        Run agent instruction
        :param request:
        :return:
        """
        async for line in self._stream("/api/agent/run", data=request.model_dump()):
            yield TaskStream.model_validate_json(line)

    async def run_agent_instruction_async(self, request: RunAgentInstructionRequest) -> RunAgentInstructionAsyncResponse:
        """
        Run agent instruction async
        :param request:
        :return:
        """
        response = await self._post("/api/agent/run-async", data=request.model_dump())
        return RunAgentInstructionAsyncResponse.model_validate_json(response.text)

    async def get_agent_task_stream(self, task_id: str):
        """
        Get agent task stream
        :param task_id:
        :return:
        """
        async for line in self._stream(f"/api/agent/tasks/{task_id}/stream"):
            yield TaskStream.model_validate_json(line)

    async def query_task_status(self, task_id: str) -> QueryTaskStatusResponse:
        """
        Query task status
        :param task_id:
        :return:
        """
        response = await self._get(f"/api/agent/tasks/{task_id}/status")
        return QueryTaskStatusResponse.model_validate_json(response.text)



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# pylint: disable=fixme

"""sandbox.py provides the Sandbox API"""
import base64
from io import BytesIO
from typing import Tuple, overload, TYPE_CHECKING

import httpx

from PIL import Image
from PIL.WebPImagePlugin import WebPImageFile

from lybic import dto
from lybic._api import deprecated

if TYPE_CHECKING:
    from lybic.lybic import LybicClient

class Sandbox:
    """
    Sandbox API
    """
    def __init__(self, client: "LybicClient"):
        self.client = client

    async def list(self) -> dto.SandboxListResponseDto:
        """
        List all sandboxes
        """
        self.client.logger.debug("Listing sandboxes requests")
        response = await self.client.request("GET", f"/api/orgs/{self.client.org_id}/sandboxes")
        self.client.logger.debug(f"Listing sandboxes response: {response.text}")
        return dto.SandboxListResponseDto.model_validate_json(response.text)

    @overload
    async def create(self, data: dto.CreateSandboxDto) -> dto.Sandbox: ...

    @overload
    async def create(self, **kwargs) -> dto.Sandbox: ...

    async def create(self, *args, **kwargs) -> dto.Sandbox:
        """
        Create a new sandbox
        """
        if args and isinstance(args[0], dto.CreateSandboxDto):
            data = args[0]
        elif "data" in kwargs and isinstance(kwargs["data"], dto.CreateSandboxDto):
            data = kwargs["data"]
        else:
            data = dto.CreateSandboxDto(**kwargs)
        self.client.logger.debug(f"Creating sandbox with data: {data}")
        response = await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/sandboxes", json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Create sandbox response: {response.text}")
        return dto.Sandbox.model_validate_json(response.text)

    async def get(self, sandbox_id: str) -> dto.GetSandboxResponseDto:
        """
        Get a sandbox
        """
        self.client.logger.debug(f"Get sandbox {sandbox_id}")
        response = await self.client.request(
            "GET",
            f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}")
        self.client.logger.debug(f"Get sandbox response: {response.text}")
        return dto.GetSandboxResponseDto.model_validate_json(response.text)

    async def delete(self, sandbox_id: str) -> None:
        """
        Delete a sandbox
        """
        self.client.logger.debug(f"Delete sandbox {sandbox_id}")
        await self.client.request(
            "DELETE",
            f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}")
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Lybic Sandbox has support multiple types of tools now and will no longer be limited to computer_use. "
                "To execute a single action tool, such as computer_use, use the execute_computer_use_action method "
                "in the ComputerUse class. If no distinction is made, use the `execute_sandbox_action` method in "
                "the Sandbox class,which is the recommended approach."
    )
    @overload
    async def execute_computer_use_action(self, sandbox_id: str, data: dto.ComputerUseActionDto) -> dto.SandboxActionResponseDto: ...
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Lybic Sandbox has support multiple types of tools now and will no longer be limited to computer_use. "
                "To execute a single action tool, such as computer_use, use the execute_computer_use_action method "
                "in the ComputerUse class. If no distinction is made, use the `execute_sandbox_action` method in "
                "the Sandbox class,which is the recommended approach."
    )
    @overload
    async def execute_computer_use_action(self, sandbox_id: str, **kwargs) -> dto.SandboxActionResponseDto: ...
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Lybic Sandbox has support multiple types of tools now and will no longer be limited to computer_use. "
                "To execute a single action tool, such as computer_use, use the execute_computer_use_action method "
                "in the ComputerUse class. If no distinction is made, use the `execute_sandbox_action` method in "
                "the Sandbox class,which is the recommended approach."
    )
    async def execute_computer_use_action(self, sandbox_id: str, *args, **kwargs) -> dto.SandboxActionResponseDto:
        """
        Execute a computer use action

        is same as mcp.ComputerUse.execute_computer_use_action
        """
        if args and isinstance(args[0], dto.ComputerUseActionDto):
            data = args[0]
        elif "data" in kwargs:
            data_arg = kwargs["data"]
            if isinstance(data_arg, dto.ComputerUseActionDto):
                data = data_arg
            elif isinstance(data_arg, dict):
                data = dto.ComputerUseActionDto(**data_arg)
            else:
                raise TypeError(f"The 'data' argument must be of type {dto.ComputerUseActionDto.__name__} or dict")
        else:
            data = dto.ComputerUseActionDto(**kwargs)
        self.client.logger.debug(f"Execute computer use action request: {data.model_dump_json(exclude_none=True)}")
        response = await self.client.request("POST",
                                       f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}/actions/computer-use",
                                       json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Execute computer use action response: {response.text}")
        return dto.SandboxActionResponseDto.model_validate_json(response.text)

    async def preview(self, sandbox_id: str) -> dto.SandboxActionResponseDto:
        """
        Preview a sandbox
        """
        self.client.logger.debug(f"Previewing sandbox {sandbox_id}")
        response = await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}/preview")
        self.client.logger.debug(f"Previewed sandbox {sandbox_id}")
        return dto.SandboxActionResponseDto.model_validate_json(response.text)

    async def extend_life(self, sandbox_id: str, seconds: int = 3600) -> None:
        """Extend the life of a sandbox.

        Args:
            sandbox_id: The ID of the sandbox to extend.
            seconds: The duration in seconds to extend the sandbox's life.
                     Default is 3600 (1 hour), max is 86400 (1 day).
                     The new max life time of the sandbox (relative to the current time) in seconds. Should not less
                     than 30 seconds or more than 24 hours. Note that the total maximum lifetime of a sandbox should
                     not longer than 13 days.
        """
        self.client.logger.debug(f"Extending life of sandbox {sandbox_id}")
        data = dto.ExtendSandboxDto(maxLifeSeconds=seconds)
        await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}/extend",
            json=data.model_dump(exclude_none=True))

    async def get_connection_details(self, sandbox_id: str)-> dto.ConnectDetails:
        """
        Get stream connection details for a sandbox
        """
        sandbox = await self.get(sandbox_id)
        return sandbox.connectDetails

    async def get_screenshot(self, sandbox_id: str) -> Tuple[str, Image.Image, str]:
        """
        Get screenshot of a sandbox

        Return screenShot_url, screenshot_image, base64_str(utf-8 encode)
        """
        result = await self.preview(sandbox_id)
        screenshot_url = result.screenShot

        async with httpx.AsyncClient() as client:
            screenshot_response = await client.get(
                screenshot_url,
                timeout=self.client.timeout
            )
            screenshot_response.raise_for_status()

            img = Image.open(BytesIO(screenshot_response.content))
            base64_str=''

            if isinstance(img, WebPImageFile):
                buffer = BytesIO()
                img.save(buffer, format="WebP")
                base64_str = base64.b64encode(buffer.getvalue()).decode("utf-8")

        return screenshot_url,img,base64_str


    async def get_screenshot_base64(self, sandbox_id: str) -> str:
        """
        Get screenshot of a sandbox in base64 format
        """
        _, _, base64_str = await self.get_screenshot(sandbox_id)
        return base64_str

    async def get_shapes(self)-> dto.GetShapesResponseDto:
        """
        Get shapes of a sandbox
        :return:
        GetShapersResponseDto
        """
        self.client.logger.debug("Getting shapes")
        response = await self.client.request(
            "GET",
            f"/api/orgs/{self.client.org_id}/shapes"
        )
        self.client.logger.debug(f"Get shapes response: {response.text}")
        return dto.GetShapesResponseDto.model_validate_json(response.text)

    @overload
    async def execute_sandbox_action(self, sandbox_id: str, data: dto.ExecuteSandboxActionDto) -> dto.SandboxActionResponseDto: ...

    @overload
    async def execute_sandbox_action(self, sandbox_id: str, **kwargs) -> dto.SandboxActionResponseDto: ...

    async def execute_sandbox_action(self, sandbox_id: str, *args, **kwargs) -> dto.SandboxActionResponseDto:
        """
        Executes a computer use or mobile use action on the sandbox.
        The action can be either a computer use or mobile use action.
        """
        if args and isinstance(args[0], dto.ExecuteSandboxActionDto):
            data = args[0]
        elif "data" in kwargs:
            data_arg = kwargs["data"]
            if isinstance(data_arg, dto.ExecuteSandboxActionDto):
                data = data_arg
            elif isinstance(data_arg, dict):
                data = dto.ExecuteSandboxActionDto(**data_arg)
            else:
                raise TypeError(f"The 'data' argument must be of type {dto.ExecuteSandboxActionDto.__name__} or dict")
        else:
            data = dto.ExecuteSandboxActionDto(**kwargs)
        self.client.logger.debug(f"Execute sandbox action request: {data.model_dump_json(exclude_none=True)}")
        response = await self.client.request("POST",
                                             f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}/actions/execute",
                                             json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Execute sandbox action response: {response.text}")
        return dto.SandboxActionResponseDto.model_validate_json(response.text)

    @overload
    async def copy_files(self, sandbox_id: str, data: dto.SandboxFileCopyRequestDto) -> dto.SandboxFileCopyResponseDto: ...

    @overload
    async def copy_files(self, sandbox_id: str, **kwargs) -> dto.SandboxFileCopyResponseDto: ...

    async def copy_files(self, sandbox_id: str, *args, **kwargs) -> dto.SandboxFileCopyResponseDto:
        """
        Copy files between sandbox and external locations (HTTP/S3).
        """
        if args and isinstance(args[0], dto.SandboxFileCopyRequestDto):
            data = args[0]
        elif "data" in kwargs:
            data_arg = kwargs["data"]
            if isinstance(data_arg, dto.SandboxFileCopyRequestDto):
                data = data_arg
            elif isinstance(data_arg, dict):
                data = dto.SandboxFileCopyRequestDto(**data_arg)
            else:
                raise TypeError(f"The 'data' argument must be of type {dto.SandboxFileCopyRequestDto.__name__} or dict")
        else:
            data = dto.SandboxFileCopyRequestDto(**kwargs)
        self.client.logger.debug(f"Copying files for sandbox {sandbox_id} with data {data.model_dump_json(exclude_none=True)}")
        response = await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}/file/copy",
            json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Copy files response: {response.text}")
        return dto.SandboxFileCopyResponseDto.model_validate_json(response.text)

    @overload
    async def execute_process(self, sandbox_id: str, data: dto.SandboxProcessRequestDto) -> dto.SandboxProcessResponseDto: ...

    @overload
    async def execute_process(self, sandbox_id: str, **kwargs) -> dto.SandboxProcessResponseDto: ...

    async def execute_process(self, sandbox_id: str, *args, **kwargs) -> dto.SandboxProcessResponseDto:
        """
        Execute a process inside sandbox.
        """
        if args and isinstance(args[0], dto.SandboxProcessRequestDto):
            data = args[0]
        elif "data" in kwargs:
            data_arg = kwargs["data"]
            if isinstance(data_arg, dto.SandboxProcessRequestDto):
                data = data_arg
            elif isinstance(data_arg, dict):
                data = dto.SandboxProcessRequestDto(**data_arg)
            else:
                raise TypeError(f"The 'data' argument must be of type {dto.SandboxProcessRequestDto.__name__} or dict")
        else:
            data = dto.SandboxProcessRequestDto(**kwargs)
        self.client.logger.debug(f"Executing process in sandbox {sandbox_id}")
        response = await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}/process",
            json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Execute process response: {response.text}")
        return dto.SandboxProcessResponseDto.model_validate_json(response.text)



"""Helper functions for managing the Lybic API.

This module is only relevant for Lybic developers, not for users.

.. warning::

    This module and its submodules are for internal use only.  Do not use them
    in your own code.  We may change the API at any time with no warning.

"""

__all__ = [
    "deprecated",
    "LybicDeprecationWarning",
    "LybicPendingDeprecationWarning",
    "warn_deprecated",
    "rename_parameter",
]

from .deprecation import (
    deprecated,
    LybicDeprecationWarning,
    LybicPendingDeprecationWarning,
    warn_deprecated,
    rename_parameter
)

def __dir__() -> list[str]:
    return list(__all__)



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""OS-level action types."""
from typing import Literal
from pydantic import BaseModel

class OsStartAppAction(BaseModel):
    """
    Start an app by its package name.
    """
    type: Literal["os:startApp"] = "os:startApp"
    packageName: str

class OsStartAppByNameAction(BaseModel):
    """
    Start an app by its name.
    """
    type: Literal["os:startAppByName"] = "os:startAppByName"
    name: str

class OsCloseAppAction(BaseModel):
    """
    Close the current app.
    """
    type: Literal["os:closeApp"] = "os:closeApp"

class OsListAppsAction(BaseModel):
    """
    List installed apps.
    """
    type: Literal["os:listApps"] = "os:listApps"



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""dto.py provides all the data types used in the API."""
import uuid
from enum import Enum, unique
from typing import List, Optional, Literal
from pydantic import BaseModel, Field, RootModel, ConfigDict
from pydantic.config import ExtraValues

from lybic._api import deprecated

# pylint: disable=invalid-name,unused-import

# Import actions from the new action module for backward compatibility
from lybic.action import (
    # Common types
    PixelLength,
    FractionalLength,
    Length,
    ClientUserTakeoverAction,
    ScreenshotAction,
    WaitAction,
    FinishedAction,
    FailedAction,
    CommonAction,

    # Computer use actions
    MouseClickAction,
    MouseTripleClickAction,
    MouseDoubleClickAction,
    MouseMoveAction,
    MouseScrollAction,
    MouseDragAction,
    KeyboardTypeAction,
    KeyboardHotkeyAction,
    KeyDownAction,
    KeyUpAction,
    ComputerUseAction,

    # Touch actions
    TouchTapAction,
    TouchDragAction,
    TouchSwipeAction,
    TouchLongPressAction,

    # Android actions
    AndroidBackAction,
    AndroidHomeAction,

    # OS actions
    OsStartAppAction,
    OsStartAppByNameAction,
    OsCloseAppAction,
    OsListAppsAction,

    # Union types
    MobileUseAction,
    Action,
)
# Strategy for handling extra fields in the lybic api response
json_extra_fields_policy: ExtraValues = "ignore"

class StatsResponseDto(BaseModel):
    """
    Organization Stats response.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy,validate_assignment=True)
    mcpServers: int
    sandboxes: int
    projects: int


class McpServerPolicy(BaseModel):
    """
    MCP server sandbox policy.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    sandboxShape: str = Field('', description="The shape of the sandbox created by the MCP server.")
    sandboxMaxLifetimeSeconds: int = Field(3600, description="The maximum lifetime of a sandbox.")
    sandboxMaxIdleTimeSeconds: int = Field(3600, description="The maximum idle time of a sandbox.")
    sandboxAutoCreation: bool = Field(False,
                                      description="Whether to create a new sandbox automatically when old sandbox is deleted. If not, new sandboxes will be created when calling computer use tools.")
    sandboxExposeRecreateTool: bool = Field(False, description="Whether to expose recreate tool to LLMs.")
    sandboxExposeRestartTool: bool = Field(False, description="Whether to expose restart tool to LLMs.")
    sandboxExposeDeleteTool: bool = Field(False, description="Whether to expose delete tool to LLMs.")


class McpServerResponseDto(BaseModel):
    """
    MCP server response.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    id: str = Field(..., description="ID of the MCP server.")
    name: str = Field(..., description="Name of the MCP server.")
    createdAt: str = Field(..., description="Creation date of the MCP server.")
    defaultMcpServer: bool = Field(..., description="Whether this is the default MCP server for the organization.")
    projectId: str = Field(..., description="Project ID to which the MCP server belongs.")
    policy: McpServerPolicy
    currentSandboxId: Optional[str] = Field(None, description="ID of the currently connected sandbox.")

class ListMcpServerResponse(RootModel):
    """
    A list of MCP server responses.
    """
    root: List[McpServerResponseDto]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class CreateMcpServerDto(McpServerPolicy):
    """
    Create MCP server request.
    Only name is needed, other fields are optional.
    """
    name: str = Field(..., description="Name of the MCP server.")
    projectId: Optional[str] = Field('', description="Project to which the MCP server belongs to.")
    sandboxShape: str = Field('', description="The shape of the sandbox created by the MCP server.")

    sandboxMaxLifetimeSeconds: Optional[int] = Field(3600, description="The maximum lifetime of a sandbox.")
    sandboxMaxIdleTimeSeconds: Optional[int] = Field(3600, description="The maximum idle time of a sandbox.")
    sandboxAutoCreation: Optional[bool] = Field(False,
                                                description="Whether to create a new sandbox automatically when old sandbox is deleted. If not, new sandboxes will be created when calling computer use tools.")
    sandboxExposeRecreateTool: Optional[bool] = Field(False, description="Whether to expose recreate tool to LLMs.")
    sandboxExposeRestartTool: Optional[bool] = Field(False, description="Whether to expose restart tool to LLMs.")
    sandboxExposeDeleteTool: Optional[bool] = Field(False, description="Whether to expose delete tool to LLMs.")

class Shape(BaseModel):
    """
    Represents a shape of a sandbox.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    name: str = Field(..., description="Name of the shape.")
    description: str = Field(..., description="Description of the shape.")
    hardwareAcceleratedEncoding: bool = Field(False, description="Whether the shape supports hardware accelerated encoding.")
    pricePerHour: float = Field(..., description="This price acts as a multiplier, e.g. if it is set to 0.5, each hour of usage will be billed as 0.5 hours.")
    requiredPlanTier: float = Field(..., description="Required plan tier to use this shape.")
    os: Literal["Windows","Linux","Android"]
    virtualization: Literal["KVM","Container"]
    architecture: Literal["x86_64","aarch64"]

# Sandbox Schemas
class Sandbox(BaseModel):
    """
    Represents a sandbox environment.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    id: str = Field(..., description="ID of the sandbox.")
    name: str = Field(..., description="Name of the sandbox.")
    expiredAt: str = Field(..., description="Deprecated, use `expiresAt` instead, will be removed in v1.0.0")
    expiresAt: str = Field(..., description="Expiration date of the sandbox.")
    createdAt: str = Field(..., description="Creation date of the sandbox.")
    projectId: str = Field(..., description="Project ID to which the sandbox belongs.")
    shapeName: Optional[str] = Field(None, description="Specs and datacenter of the sandbox.") # This field does not exist in GetSandboxResponseDto (that is, this field is optional)
    shape: Optional[Shape] = None # This field does not exist in SandboxListResponseDto (that is, this field is optional)


class GatewayAddress(BaseModel):
    """
    Details of a gateway address for connecting to a sandbox.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    address: str
    port: int
    name: str
    preferredProviders: List[Literal["CHINA_MOBILE", "CHINA_UNICOM", "CHINA_TELECOM", "GLOBAL_BGP", 1, 2, 3, 4]]
    gatewayType: Literal["KCP", "QUIC", "WEB_TRANSPORT", "WEBSOCKET","WEBSOCKET_SECURE", 4, 5, 6, 7, 8]
    path: Optional[str] = None


class ConnectDetails(BaseModel):
    """
    Connection details for a sandbox, including gateway addresses and authentication tokens.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    gatewayAddresses: List[GatewayAddress]
    certificateHashBase64: str
    endUserToken: str
    roomId: str


class SandboxListItem(Sandbox):
    """
    An item in a list of sandboxes, containing sandbox details and connection information.
    """


class SandboxListResponseDto(RootModel):
    """
    A response DTO containing a list of sandboxes.
    """
    root: List[SandboxListItem]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class CreateSandboxDto(BaseModel):
    """
    Create sandbox request.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    name: str = Field("sandbox", description="The name of the sandbox.")
    maxLifeSeconds: int = Field(3600,
                                description="The maximum life time of the sandbox in seconds. Default is 1 hour, max is 1 day.",
                                ge=1, le=86400)
    projectId: Optional[str] = Field(None, description="The project id to use for the sandbox. Use default if not provided.")
    shape: str = Field(..., description="Specs and datacenter of the sandbox.")


class GetSandboxResponseDto(BaseModel):
    """
    A response DTO for a single sandbox, including connection details.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    sandbox: Sandbox
    connectDetails: ConnectDetails


# Computer Use Schemas
# (Actions moved to lybic.action module for better organization)


@deprecated(
    since="0.8.0",
    removal="1.0.0",
    message="Use `ExecuteSandboxActionDto` instead, which supports both computer and mobile use actions."
)
class ComputerUseActionDto(BaseModel):
    """
    Computer use action request.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    action: ComputerUseAction | dict
    includeScreenShot: bool = True
    includeCursorPosition: bool = True
    callId: Optional[str] = Field(default_factory=lambda: str(uuid.uuid4()))


class CursorPosition(BaseModel):
    """
    Represents the position of the cursor on the screen.
    """
    x: int
    y: int
    screenWidth: int
    screenHeight: int
    screenIndex: int

class ExtendSandboxDto(BaseModel):
    """
    Extend sandbox life request.
    """
    maxLifeSeconds: int = Field(3600, description="Max life seconds of sandbox", ge=30, le=60 * 60 * 24)


class SandboxActionResponseDto(BaseModel):
    """
    Computer use action response.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    screenShot: Optional[str] = None  # is a picture url of the screen eg. https://example.com/screen.webp
    cursorPosition: Optional[CursorPosition] = None
    actionResult: Optional[str] = None

@unique
class ModelType(Enum):
    """
    Enumeration of supported LLM models for computer-use parsing.
    """
    UITARS = "ui-tars"
    SEED = "seed"
    GLM_4_1V = "glm-4.1v"
    GLM_4_5V = "glm-4.5v"
    QWEN_2_5_VL = "qwen-2.5-vl"
    PYAUTOGUI = "pyautogui"


class ParseTextRequestDto(BaseModel):
    """
    Request DTO for parsing text content.
    """
    textContent: str


@deprecated(
    since="0.7.0",
    removal="1.0.0",
    message=(
        "Starting from v0.7.0, parsing LLM output functions(ComputerUse.parse_llm_output) will "
        "no longer require ComputerUseParseRequestDto"
    )
)
class ComputerUseParseRequestDto(BaseModel):
    """
    Request DTO for parsing text content into computer use actions.
    """
    model: Literal["ui-tars", "oai-compute-use", "seed"]
    textContent: str


class ComputerUseActionResponseDto(BaseModel):
    """
    Response DTO containing a list of parsed computer use actions.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    unknown: Optional[str] = None
    thoughts: Optional[str] = None
    memory: Optional[str] = None

    actions: List[ComputerUseAction]


# Mobile Use Schemas
# (Actions moved to lybic.action module for better organization)


class ExecuteSandboxActionDto(BaseModel):
    """
    Sandbox action request, supporting both computer and mobile use actions.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    action: Action | dict
    includeScreenShot: bool = True
    includeCursorPosition: bool = True
    callId: Optional[str] = Field(default_factory=lambda: str(uuid.uuid4()))


# Project Schemas
class ProjectResponseDto(BaseModel):
    """
    Get Project Response
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    id: str
    name: str
    createdAt: str
    defaultProject: bool


class ListProjectsResponseDto(RootModel):
    """
    A response DTO containing a list of projects.
    """
    root: List[ProjectResponseDto]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class CreateProjectDto(BaseModel):
    """
    Data transfer object for creating a new project.
    """
    name: str


class SingleProjectResponseDto(ProjectResponseDto):
    """
    Response DTO for a single project.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)


class SetMcpServerToSandboxResponseDto(BaseModel):
    """
    Response DTO for setting a MCP server to a sandbox.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    sandboxId: Optional[str] = Field(None, description="The ID of the sandbox to connect the MCP server to.")


class Shapes(BaseModel):
    """
    Shapes
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    name: str
    description: str
    hardwareAcceleratedEncoding: bool
    pricePerHour: str
    requiredPlanTier: int
    os: str
    virtualization:  str
    architecture:  str


class GetShapesResponseDto(RootModel):
    """
    Response DTO for getting shapers.
    """
    root: List[Shapes]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class MobileUseActionResponseDto(BaseModel):
    """
    Response DTO containing a list of parsed mobile use actions.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    unknown: Optional[str] = None
    thoughts: Optional[str] = None
    memory: Optional[str] = None
    actions: List[MobileUseAction]



# File Transfer Schemas
class SandboxFileLocation(BaseModel):
    """
    Sandbox file location.
    """
    type: Literal["sandboxFileLocation"] = "sandboxFileLocation"
    path: str = Field(..., min_length=1, description="File path in sandbox")


class HttpPutLocation(BaseModel):
    """
    HTTP PUT location.
    """
    type: Literal["httpPutLocation"] = "httpPutLocation"
    url: str = Field(..., description="PUT upload URL")
    headers: Optional[dict] = Field(default=None, description="Optional HTTP headers")


class HttpGetLocation(BaseModel):
    """
    HTTP GET location.
    """
    type: Literal["httpGetLocation"] = "httpGetLocation"
    url: str = Field(..., description="GET download URL")
    headers: Optional[dict] = Field(default=None, description="Optional HTTP headers")


class HttpPostFormLocation(BaseModel):
    """
    HTTP POST form location.
    """
    type: Literal["httpPostFormLocation"] = "httpPostFormLocation"
    url: str = Field(..., description="POST form upload URL")
    form: dict = Field(..., description="Form fields")
    fileField: str = Field(default="file", description="File field name in form")
    headers: Optional[dict] = Field(default=None, description="Optional HTTP headers")


# Union type for file locations
FileLocation = SandboxFileLocation | HttpPutLocation | HttpGetLocation | HttpPostFormLocation


class FileCopyItem(BaseModel):
    """
    Single file copy item.
    """
    id: Optional[str] = Field(None, description="A caller-defined unique identifier for this item. The value is included in the response to associate results with their corresponding requests")
    src: FileLocation = Field(..., description="Copy file source")
    dest: FileLocation = Field(..., description="Copy file destination")

class SandboxFileCopyRequestDto(BaseModel):
    """
    Request DTO for copying files with sandbox.
    """
    files: List[FileCopyItem] = Field(..., min_length=1)


class FileCopyResult(BaseModel):
    """
    Single file copy result.
    """
    id: Optional[str] = Field(None, description="Unique identifier of the files item from the request")
    success: bool = Field(..., description="Whether the operation succeeded")
    error: Optional[str] = Field(None, description="Error message if failed")


class SandboxFileCopyResponseDto(BaseModel):
    """
    Response DTO for file copy operation.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    results: List[FileCopyResult]

# Process Execution Schemas
class SandboxProcessRequestDto(BaseModel):
    """
    Request DTO for executing a process in sandbox.
    """
    executable: str = Field(..., min_length=1, description="Executable path")
    args: List[str] = Field(default_factory=list, description="Arguments")
    workingDirectory: Optional[str] = Field(None, description="Working directory")
    stdinBase64: Optional[str] = Field(None, description="Optional stdin as base64-encoded bytes")


class SandboxProcessResponseDto(BaseModel):
    """
    Response DTO for process execution.
    """
    model_config = ConfigDict(extra=json_extra_fields_policy)

    stdoutBase64: str = Field(default="", description="stdout as base64-encoded bytes")
    stderrBase64: str = Field(default="", description="stderr as base64-encoded bytes")
    exitCode: int = Field(..., description="Exit code")



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""base.py holds the base client for Lybic API."""
import logging
import os
import warnings
from sys import stderr
from typing import Optional

from lybic.authentication import LybicAuth

_sentinel = object()

class _LybicBaseClient:
    """_LybicBaseClient is a base client for all Lybic API."""

    def __init__(self,
                 auth: Optional[LybicAuth] = None,
                 org_id: str = _sentinel,
                 api_key: str = _sentinel,
                 endpoint: str = _sentinel,
                 timeout: int = 10,
                 extra_headers: dict = _sentinel,
                 max_retries: int = 3,
                 ):
        """
        Init lybic client with org_id, api_key and endpoint

        :param auth: LybicAuth instance
        :param org_id:
        :param api_key:
        :param endpoint:
        """
        if auth:
            self.auth = auth
        else:
            user_provided_auth_params = any(
                val is not _sentinel for val in (org_id, api_key, endpoint, extra_headers)
            )
            if user_provided_auth_params:
                warnings.warn(
                    "Passing `org_id`, `api_key`, `endpoint`, or `extra_headers` to the client constructor is deprecated "
                    "and will be removed in v1.0.0. Please use `LybicClient(auth=LybicAuth(...))` instead.",
                    stacklevel=3
                )

            if org_id is _sentinel:
                org_id = os.getenv("LYBIC_ORG_ID")
            if api_key is _sentinel:
                api_key = os.getenv("LYBIC_API_KEY")
            if endpoint is _sentinel:
                endpoint = os.getenv("LYBIC_API_ENDPOINT", "https://api.lybic.cn")
            if extra_headers is _sentinel:
                extra_headers = None
            assert org_id, "LYBIC_ORG_ID is required"
            assert endpoint, "LYBIC_API_ENDPOINT is required"

            self.auth = LybicAuth(
                org_id=org_id,
                api_key=api_key,
                endpoint=endpoint,
                extra_headers=extra_headers
            )

        if timeout < 0:
            print("Warning: Timeout cannot be negative, set to 10", file=stderr)
            timeout = 10
        self.timeout = timeout
        self.max_retries = max(max_retries, 0)

        self.logger = logging.getLogger(__name__)

    @property
    def headers(self):
        """
        Get headers for requests

        :return:
        """
        return self.auth.headers

    @property
    def endpoint(self):
        """
        Get endpoint for requests

        :return:
        """
        return self.auth.endpoint

    @property
    def org_id(self):
        """
        Get org_id for requests

        :return:
        """
        return self.auth.org_id

    @property
    def _api_key(self):
        return self.auth.api_key

    def make_mcp_endpoint(self, mcp_server_id: str) -> str:
        """
        Make MCP endpoint for a MCP server

        :param mcp_server_id:
        :return:
        """
        return f"{self.endpoint}/api/mcp/{mcp_server_id}"



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""authentication.py holds the authentication for Lybic API."""
import os


class LybicAuth:
    """LybicAuth holds the authentication for Lybic API."""

    org_id:  str # Your organization ID
    api_key: str # Your API key
    endpoint: str # Your API endpoint
    headers: dict # Extra headers if needed

    agent_service_endpoint: str

    def __init__(self,
                 org_id: str = os.getenv("LYBIC_ORG_ID"),
                 api_key: str = os.getenv("LYBIC_API_KEY"),
                 endpoint: str = os.getenv("LYBIC_API_ENDPOINT", "https://api.lybic.cn"),
                 agent_service_endpoint: str = os.getenv("LYBIC_AGENT_SERVICE_ENDPOINT", "https://agent.lybic.cn"),
                 extra_headers: dict = None,
                 ):
        """
        Initializes the LybicAuth instance.

        :param org_id: Your organization ID. Defaults to the `LYBIC_ORG_ID` environment variable. Required.
        :param api_key: Your API key. Defaults to the `LYBIC_API_KEY` environment variable. Required unless `x-trial-session-token` is in `extra_headers`.
        :param endpoint: The API endpoint. Defaults to the `LYBIC_API_ENDPOINT` environment variable or "https://api.lybic.cn". Required.
        :param extra_headers: A dictionary of extra headers to include in requests.
        """
        self.headers = {}
        if extra_headers:
            self.headers.update(extra_headers)

        # if x-trial-session-token is provided, use it instead of api_key
        if not (extra_headers and 'x-trial-session-token' in extra_headers):
            assert api_key, "LYBIC_API_KEY is required when x-trial-session-token is not provided"
            self.headers["x-api-key"] = api_key
        self.api_key = api_key

        self.endpoint = endpoint.rstrip('/')
        self.agent_service_endpoint = agent_service_endpoint.rstrip('/')

        self.org_id = org_id
        self.headers["Content-Type"] = "application/json"



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Computer use action types."""
from typing import Literal, Optional, Union
from pydantic import BaseModel, Field

from .common import Length, ScreenshotAction, WaitAction, FinishedAction, FailedAction, ClientUserTakeoverAction

class MouseClickAction(BaseModel):
    """
    Represents a mouse click action at a specified location.
    """
    type: Literal["mouse:click"] = "mouse:click"
    x: Length
    y: Length
    button: int = Field(..., description="Mouse button flag combination. 1: left, 2: right, 4: middle, 8: back, 16: forward; add them together to press multiple buttons at once.")
    relative: bool = Field(False, description="Whether the coordinates are relative to the current mouse position")
    holdKey: Optional[str] = Field(None, description="Key to hold down during click, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")


class MouseTripleClickAction(BaseModel):
    """
    Represents a mouse triple-click action at a specified location.
    """
    type: Literal["mouse:tripleClick"] = "mouse:tripleClick"
    x: Length
    y: Length
    button: int = Field(..., description="Mouse button flag combination. 1: left, 2: right, 4: middle, 8: back, 16: forward; add them together to press multiple buttons at once.")
    relative: bool = Field(False, description="Whether the coordinates are relative to the current mouse position.")
    holdKey: Optional[str] = Field(None, description="Key to hold down during triple click, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")


class MouseDoubleClickAction(BaseModel):
    """
    Represents a mouse double-click action at a specified location.
    """
    type: Literal["mouse:doubleClick"] = "mouse:doubleClick"
    x: Length
    y: Length
    button: int = Field(..., description="Mouse button flag combination. 1: left, 2: right, 4: middle, 8: back, 16: forward; add them together to press multiple buttons at once.")
    relative: bool = Field(False, description="Whether the coordinates are relative to the current mouse position")
    holdKey: Optional[str] = Field(None, description="Key to hold down during click, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")



class MouseMoveAction(BaseModel):
    """
    Represents a mouse move action to a specified location.
    """
    type: Literal["mouse:move"] = "mouse:move"
    x: Length
    y: Length
    relative: bool = Field(False, description="Whether the coordinates are relative to the current mouse position")
    holdKey: Optional[str] = Field(None, description="Key to hold down during move, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")


class MouseScrollAction(BaseModel):
    """
    Represents a mouse scroll action.
    """
    type: Literal["mouse:scroll"] = "mouse:scroll"
    x: Length
    y: Length
    stepVertical: int
    stepHorizontal: int
    relative: bool = Field(False, description="Whether the coordinates are relative to the current mouse position")
    holdKey: Optional[str] = Field(None, description="Key to hold down during scroll, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")


class MouseDragAction(BaseModel):
    """
    Represents a mouse drag action from a start to an end point.
    """
    type: Literal["mouse:drag"] = "mouse:drag"
    startX: Length
    startY: Length
    endX: Length
    endY: Length
    startRelative: bool = Field(False, description="Whether the start coordinates are relative to the current mouse position.")
    endRelative: bool = Field(False, description="Whether the end coordinates are relative to the start coordinates of the drag. If false, they are absolute screen coordinates.")
    button: int = Field(..., description="Mouse button flag combination. 1: left, 2: right, 4: middle, 8: back, 16: forward; add them together to press multiple buttons at once.")
    holdKey: Optional[str] = Field(None, description="Key to hold down during drag, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")


class KeyboardTypeAction(BaseModel):
    """
    Represents a keyboard typing action.
    """
    type: Literal["keyboard:type"] =  "keyboard:type"
    content: str
    treatNewLineAsEnter: bool = Field(False, description="Whether to treat line breaks as enter. If true, any line breaks(\\n) in content will be treated as enter key press, and content will be split into multiple lines.")


class KeyboardHotkeyAction(BaseModel):
    """
    Represents a keyboard hotkey combination action.
    """
    type: Literal["keyboard:hotkey"] =  "keyboard:hotkey"
    keys: str
    duration: Optional[int] = Field(None, description="Duration in milliseconds. If specified, the hotkey will be held for a while and then released.")

class KeyDownAction(BaseModel):
    """
    Press ONE key down, in xdotool key syntax. Only use this action if hotkey or type cannot satisfy your needs.
    """
    type: Literal["key:down"]= "key:down"
    key: str


class KeyUpAction(BaseModel):
    """
    Release ONE key, in xdotool key syntax. Only use this action if keydown cannot satisfy your needs and only after a key down.
    """
    type: Literal["key:up"] = "key:up"
    key: str

ComputerUseAction = Union[
    MouseClickAction,
    MouseTripleClickAction,
    MouseDoubleClickAction,
    MouseMoveAction,
    MouseScrollAction,
    MouseDragAction,

    KeyboardTypeAction,
    KeyboardHotkeyAction,
    KeyDownAction,
    KeyUpAction,

    ScreenshotAction,
    WaitAction,
    FinishedAction,
    FailedAction,
    ClientUserTakeoverAction,
]



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""exceptions.py provides custom exceptions for Lybic API."""
from typing import Optional


class LybicError(Exception):
    """Base exception class for all Lybic SDK errors."""

    def __init__(self, message: str, status_code: Optional[int] = None):
        """
        Initialize LybicError.

        :param message: Error message
        :param status_code: HTTP status code if applicable
        """
        self.message = message
        self.status_code = status_code
        super().__init__(message)


class LybicAPIError(LybicError):
    """Exception raised for API errors with structured error responses.
    
    This exception is raised when the API returns an error response with
    a structured format containing 'code' and 'message' fields.
    
    Example response:
    {"code": "nomos.partner.NO_ROOMS_AVAILABLE", "message": "No rooms available"}
    """

    def __init__(
        self,
        message: str,
        code: Optional[str] = None,
        status_code: Optional[int] = None,
    ):
        """
        Initialize LybicAPIError.

        :param message: Error message from API response
        :param code: Error code from API response
        :param status_code: HTTP status code
        """
        self.code = code
        super().__init__(message, status_code)

    def __str__(self) -> str:
        """Return string representation of the error."""
        if self.code:
            return f"{self.message} (code: {self.code})"
        return self.message


class LybicInternalError(LybicError):
    """Exception raised for internal server errors (5xx) from reverse proxy.
    
    This exception is raised when a 5xx error occurs at the reverse proxy level,
    typically returning an HTML error page instead of a JSON response.
    """

    def __init__(self, status_code: int = 500):
        """
        Initialize LybicInternalError.

        :param status_code: HTTP status code (5xx)
        """
        super().__init__("internal error occur", status_code)



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

""" Lybic SDK import header"""

__version__ = "0.12.0" # SDK Api Leval

# Strategy for handling extra fields in the lybic api response
# "ignore" means ignore extra fields, which will ensure that your SDK version remains compatible with the Lybic platform,
# but it may cause compatibility issues with future versions of the SDK.
# "forbid" means that the SDK will throw an error if it encounters extra fields in the response, which will force you to
# update your SDK when the Lybic platform is updated, and may have a certain impact on your online environment.
from .dto import json_extra_fields_policy

# Lybic Client
from .authentication import LybicAuth
from .lybic import LybicClient

# Exceptions
from .exceptions import LybicError, LybicAPIError, LybicInternalError

# MCP Operations
from .mcp import MCP, Mcp

#  Tools
from .tools import ComputerUse, MobileUse, Tools

# Project
from .project import Project

# Pyautogui
from .pyautogui import Pyautogui

# Sandbox
from .sandbox import Sandbox

# Stats
from .stats import Stats

__all__ = [
    "__version__",
    "LybicAuth",
    "LybicClient",

    "LybicError",
    "LybicAPIError",
    "LybicInternalError",

    "MCP",
    "ComputerUse",
    "Project",
    "Pyautogui",
    "Sandbox",
    "Stats",

    "json_extra_fields_policy",
]

def __dir__() -> list[str]:
    return list(__all__)

# pylint: disable=invalid-name
version = __version__  # SDK Api Level



"""
lmcp.py has been moved to mcp.py

And it will be removed in the 1.0 stable version, please modify your import

from lybic.lmcp import MCP,ComputerUse
 ->
from lybic.mcp import MCP,ComputerUse

or

from lybic import MCP,ComputerUse
"""
# pylint: disable=unused-import

import warnings
from .mcp import MCP
from .tools import ComputerUse

warnings.warn(
    "The 'lybic.lmcp' module has been moved to 'lybic.mcp' and is deprecated. It will be removed in version 1.0. Please update your imports.",
    DeprecationWarning,
    stacklevel=2
)



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Common action types and helper types."""
from typing import Literal, Optional, Union
from pydantic import BaseModel

class PixelLength(BaseModel):
    """
    Represents a length in pixels.
    """
    type: Literal["px"] = "px"
    value: int


class FractionalLength(BaseModel):
    """
    Represents a length as a fraction of a total dimension.
    """
    type: Literal["/"] = "/"
    numerator: int
    denominator: int

Length = Union[PixelLength, FractionalLength]


class ClientUserTakeoverAction(BaseModel):
    """
    Indicates the human user should take over the control.
    """
    type: Literal["client:user-takeover"] = "client:user-takeover"


class ScreenshotAction(BaseModel):
    """
    Represents an action to take a screenshot.
    """
    type: Literal["screenshot"] = "screenshot"


class WaitAction(BaseModel):
    """
    Represents a wait action for a specified duration.
    """
    type: Literal["wait"] = "wait"
    duration: int


class FinishedAction(BaseModel):
    """
    Represents a finished action, signaling successful completion of a task.
    """
    type: Literal["finished"] = "finished"
    message: Optional[str] = None


class FailedAction(BaseModel):
    """
    Represents a failed action, signaling an error or failure in a task.
    """
    type: Literal["failed"] = "failed"
    message: Optional[str] = None


CommonAction = Union[
    ScreenshotAction,
    WaitAction,
    FinishedAction,
    FailedAction,
    ClientUserTakeoverAction,
]



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""
lybic.tools:
ComputerUse tools
MobileUse tools
"""
from typing import overload, TYPE_CHECKING

from lybic.dto import (
    ParseTextRequestDto,
    ComputerUseParseRequestDto,
    ComputerUseActionResponseDto,
    ModelType,
    ComputerUseActionDto,
    SandboxActionResponseDto,
    MobileUseActionResponseDto,
)
from lybic._api import deprecated

if TYPE_CHECKING:
    from lybic.lybic import LybicClient

class ComputerUse:
    """ComputerUse is an async client for lybic ComputerUse API(MCP and Restful)."""
    def __init__(self, client: "LybicClient"):
        self.client = client
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Use parse_llm_output instead"
    )
    @overload
    async def parse_model_output(self, data: ComputerUseParseRequestDto) -> ComputerUseActionResponseDto: ...
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Use parse_llm_output instead"
    )
    @overload
    async def parse_model_output(self, **kwargs) -> ComputerUseActionResponseDto: ...
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Use parse_llm_output instead"
    )
    async def parse_model_output(self, *args, **kwargs) -> ComputerUseActionResponseDto:
        """
        parse doubao-ui-tars output

        :param data:
        :return:
        """
        if args and isinstance(args[0], ComputerUseParseRequestDto):
            data = args[0]
        elif "data" in kwargs and isinstance(kwargs["data"], ComputerUseParseRequestDto):
            data = kwargs["data"]
        else:
            data = ComputerUseParseRequestDto(**kwargs)
        self.client.logger.debug(f"Parse model output request: {data.model_dump_json(exclude_none=True)}")
        response = await self.client.request(
            "POST",
            "/api/computer-use/parse",
            json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Parse model output response: {response.text}")
        return ComputerUseActionResponseDto.model_validate_json(response.text)
    async def parse_llm_output(
        self, model_type: ModelType | str, llm_output: str
    ) -> ComputerUseActionResponseDto:
        """Parse LLM output to computer use actions.

        Args:
            model_type: The type of the large language model.
            llm_output: The text output from the large language model.

        Returns:
            A DTO containing the parsed computer use actions.
        """
        if isinstance(model_type, ModelType):
            model = model_type.value
        elif isinstance(model_type, str):
            valid_models = [item.value for item in ModelType]
            if model_type not in valid_models:
                raise ValueError(f"Invalid model_type: {model_type}. Must be one of {valid_models}")
            model = model_type
        else:
            raise TypeError("model_type must be either dto.ModelType or str")

        response = await self.client.request(
            "POST",
            f"/api/computer-use/parse/{model}",
            json=ParseTextRequestDto(textContent=llm_output).model_dump(exclude_none=True),
        )
        self.client.logger.debug(f"Parse model output response: {response.text}")
        return ComputerUseActionResponseDto.model_validate_json(response.text)

    @deprecated(
        since="0.8.0",
        removal="1.0.0",
        message="Use `lybic.sandbox.Sandbox.execute_sandbox_action` instead."
    )
    @overload
    async def execute_computer_use_action(self, sandbox_id: str,
                                    data: ComputerUseActionDto) -> SandboxActionResponseDto: ...
    @deprecated(
        since="0.8.0",
        removal="1.0.0",
        message="Use `lybic.sandbox.Sandbox.execute_sandbox_action` instead."
    )
    @overload
    async def execute_computer_use_action(self, sandbox_id: str, **kwargs) -> SandboxActionResponseDto: ...

    @deprecated(
        since="0.8.0",
        removal="1.0.0",
        message="Use `lybic.sandbox.Sandbox.execute_sandbox_action` instead."
    )
    async def execute_computer_use_action(self, sandbox_id: str, *args, **kwargs) -> SandboxActionResponseDto:
        """Executes a computer use action in a specific sandbox.

        Note: This method provides the same functionality as
        `lybic.sandbox.Sandbox.execute_computer_use_action`.

        Args:
            sandbox_id: The ID of the sandbox to execute the action in.
            *args: Supports passing `dto.ComputerUseActionDto` as a positional argument.
            **kwargs: Supports passing `data` as a `dto.ComputerUseActionDto` or a `dict`,
                or the fields of `dto.ComputerUseActionDto` directly as keyword arguments.

        Returns:
            A `dto.SandboxActionResponseDto` containing the result of the action.

        Raises:
            TypeError: If the 'data' argument is not of the expected type.
        """
        if args and isinstance(args[0], ComputerUseActionDto):
            data = args[0]
        elif "data" in kwargs:
            data_arg = kwargs["data"]
            if isinstance(data_arg, ComputerUseActionDto):
                data = data_arg
            elif isinstance(data_arg, dict):
                data = ComputerUseActionDto(**data_arg)
            else:
                raise TypeError(f"The 'data' argument must be of type {ComputerUseActionDto.__name__} or dict")
        else:
            data = ComputerUseActionDto(**kwargs)
        self.client.logger.debug(f"Execute computer use action request: {data.model_dump_json(exclude_none=True)}")
        response = await self.client.request("POST",
                                       f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}/actions/computer-use",
                                       json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Execute computer use action response: {response.text}")
        return SandboxActionResponseDto.model_validate_json(response.text)

class MobileUse:
    """MobileUse is an async client for lybic MobileUse API(MCP and Restful)."""
    def __init__(self, client: "LybicClient"):
        self.client = client

    async def parse_llm_output(
        self, model_type: ModelType | str, llm_output: str
    ) -> MobileUseActionResponseDto:
        """Parse LLM output to mobile use actions.

        Args:
            model_type: The type of the large language model.
            llm_output: The text output from the large language model.

        Returns:
            A DTO containing the parsed mobile use actions.
        """
        if isinstance(model_type, ModelType):
            model = model_type.value
        elif isinstance(model_type, str):
            valid_models = [item.value for item in ModelType]
            if model_type not in valid_models:
                raise ValueError(f"Invalid model_type: {model_type}. Must be one of {valid_models}")
            model = model_type
        else:
            raise TypeError("model_type must be either dto.ModelType or str")

        response = await self.client.request(
            "POST",
            f"/api/mobile-use/parse/{model}",
            json=ParseTextRequestDto(textContent=llm_output).model_dump(exclude_none=True),
        )
        self.client.logger.debug(f"Parse model output response: {response.text}")
        return MobileUseActionResponseDto.model_validate_json(response.text)

class Tools:
    """Tools is a container for various tool clients."""
    def __init__(self, client: "LybicClient"):
        self.computer_use = ComputerUse(client)
        self.mobile_use = MobileUse(client)
